package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.jirbo.adcolony.AdColony;
import com.jirbo.adcolony.AdColonyAd;
import com.jirbo.adcolony.AdColonyAdAvailabilityListener;
import com.jirbo.adcolony.AdColonyAdListener;
import com.jirbo.adcolony.AdColonyV4VCAd;
import com.jirbo.adcolony.AdColonyV4VCListener;
import com.jirbo.adcolony.AdColonyV4VCReward;

import java.util.Map;

public class AdColonyNetwork implements AdNetwork, AdColonyAdListener, AdColonyV4VCListener,
        AdColonyAdAvailabilityListener, Application.ActivityLifecycleCallbacks {

    private static final String TAG = AdColonyNetwork.class.getName();
    private AdColonyV4VCAd v4vc_ad = null;
    private String activityClassName = null;
    private Context context = null;
    private PokktConfig pokktConfig = null;
    private Network network = null;

    /*
     * We recommend passing the AdColony client options, app ID, all zone IDs, and current zone ID
     * in the serverExtras Map by specifying Custom Network Data in Pokkt's web interface.
     *
     */
    private static final String DEFAULT_CLIENT_OPTIONS = "version=1.0,store:google";
    private static final String DEFAULT_APP_ID = "app185a7e71e1714831a49ec7";
    private static final String[] DEFAULT_ALL_ZONE_IDS = {"vz1fd5a8b2bf6841a0a4b826"};
    private static final String DEFAULT_ZONE_ID = "vz1fd5a8b2bf6841a0a4b826";

    /*
     * These keys are intended for AdColony internal use. Do not modify.
     */
    public static final String CLIENT_OPTIONS_KEY = "clientOptions";
    public static final String APP_ID_KEY = "appId";
    public static final String ALL_ZONE_IDS_KEY = "allZoneIds";
    public static final String ZONE_ID_KEY = "zoneId";

    private boolean isInitialized = false;

    @Override
    public void initNetwork(final Context context, final PokktConfig pokktConfig, final Network network) throws Exception {
        Log.d(TAG, "AdColony init network called!!");
        this.pokktConfig = pokktConfig;

        synchronized (AdColonyNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "AdColony Init Network Already Initialized!!");
                return;
            }
            this.context = context;
            this.activityClassName = ((Activity) context).getClass().getName();
            this.network = network;
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

            String adColonyClientOptions = DEFAULT_CLIENT_OPTIONS;
            String adColonyAppId = DEFAULT_APP_ID;
            String[] adColonyAllZoneIds = DEFAULT_ALL_ZONE_IDS;

            // Set up serverExtras
            if (extrasAreValid(network)) {
                Log.d(TAG, "AdColony Init Network Extras Are Valid!!");
                adColonyClientOptions = network.getCustomData().get(CLIENT_OPTIONS_KEY);
                adColonyAppId = network.getCustomData().get(APP_ID_KEY);
                adColonyAllZoneIds = extractAllZoneIds(network.getCustomData());
            } else {
                throw new Exception("AdColony Init Configurations Error!");
            }

            Log.d(TAG, "AdColony Init Network Calling Configure !!");
            // Configure ADC once early before any other ADC calls (except setCustomID/setDeviceID).
            AdColony.configure(((Activity) context), adColonyClientOptions, adColonyAppId, adColonyAllZoneIds);
            // version - arbitrary application version
            // store   - google or amazon

            // Disable rotation if not on a tablet-sized device (note: not
            // necessary to use AdColony).
            if (!AdColony.isTablet()) {
                Log.d(TAG, "AdColony Init Network Table Config!!");
                ((Activity) context).setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
            }

            Log.d(TAG, "AdColony Init Network Adding v4vc Listener!!");
            // Notify this object about confirmed virtual currency.
            AdColony.addV4VCListener(this);

            Log.d(TAG, "AdColony Init Network Adding Ad Availability Listener !!");
            // Notify this object about ad availability changes.
            AdColony.addAdAvailabilityListener(this);

            Log.d(TAG, "AdColony Init Network Initialized !!");
            isInitialized = true;
        }
    }

    private boolean extrasAreValid(Network network) {
        return network != null
                && network.getCustomData() != null
                && network.getCustomData().containsKey(CLIENT_OPTIONS_KEY)
                && network.getCustomData().containsKey(APP_ID_KEY)
                && network.getCustomData().containsKey(ALL_ZONE_IDS_KEY)
                && network.getCustomData().containsKey(ZONE_ID_KEY);
    }

    private String[] extractAllZoneIds(Map<String, String> serverExtras) {
        String zoneIds = serverExtras.get(ALL_ZONE_IDS_KEY);

        return zoneIds.split(",");
    }

    @Override
    public void cacheVideo(final Network network) {
        Log.d(TAG, "AdColony cache video called!!");
        String mZoneId = DEFAULT_ZONE_ID;
        if (extrasAreValid(network)) {
            mZoneId = network.getCustomData().get(ZONE_ID_KEY);
        }

        String status = AdColony.statusForZone(mZoneId);

        v4vc_ad = new AdColonyV4VCAd(mZoneId).withListener(this);

        Log.d(TAG, "AdColony ad zone " + mZoneId + " has status " + status + " !!");

        if ("active".equalsIgnoreCase(status)) {
            Log.d(TAG, "AdColony ad is available !!");
            int reward = v4vc_ad.getRewardAmount();
            Log.d(TAG, "AdColony ad is available and reward is " + reward);
            Log.d(TAG, "AdColony ad is available and calling download completed !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, reward, "0");
        } else {
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "AdColony Ad is Not Active", network);
        }
        Log.d(TAG, "AdColony cache video finished!!");
    }

    @Override
    public void playVideo() {
        Log.d(TAG, "AdColony play video called!!");
        Log.d(TAG, "AdColony play video called ad is not null " + (v4vc_ad != null));
        try {
            if (v4vc_ad != null) {
                Log.d(TAG, "AdColony play video called ad is ready " + v4vc_ad.isReady());
                Log.d(TAG, "AdColony play video called ad has available views " + v4vc_ad.getAvailableViews());
            }
            if (v4vc_ad != null && v4vc_ad.isReady() && v4vc_ad.getAvailableViews() != 0) {
                Log.d(TAG, "AdColony play video called as v4vc ad is available and ready !!");
                v4vc_ad.show();
            } else {
                Log.d(TAG, "AdColony play video error .. ad is not ready");
                PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
            }
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
        }
    }

    @Override
    public void onAdColonyAdAvailabilityChange(boolean available, String zone_id) {
        Log.d(TAG, "onAdColonyAdAvailabilityChange: " + available + " , " + zone_id);
        if (v4vc_ad != null) {
            if (available) {
                int reward = v4vc_ad.getRewardAmount();
                Log.d(TAG, "AdColony ad is available and reward is " + reward);
            } else {
                String reason = "No Campaigns from AdColony";
                Log.d(TAG, "Could not show ad from AdColony for reason: " + reason);
                v4vc_ad = null;
            }
        } else {
            Log.d(TAG, "AdColony is not the current network ");
        }
    }

    @Override
    //Ad Attempt Finished Callback - called at the end of any ad attempt - successful or not.
    public void onAdColonyAdAttemptFinished(AdColonyAd ad) {
        // You can ping the AdColonyAd object here for more information:
        // ad.shown() - returns true if the ad was successfully shown.
        // ad.notShown() - returns true if the ad was not shown at all (i.e. if onAdColonyAdStarted was never triggered)
        // ad.skipped() - returns true if the ad was skipped due to an interval play setting
        // ad.canceled() - returns true if the ad was cancelled (either programmatically or by the user)
        // ad.noFill() - returns true if the ad was not shown due to no ad fill.

        Log.d(TAG, "onAdColonyAdAttemptFinished");
        try {
            if (ad.shown()) {
                Log.d(TAG, "onAdColonyAdAttemptFinished ad was shown !!");
                if (ad.skipped() || ad.canceled()) {
                    Log.d(TAG, "onAdColonyAdAttemptFinished ad is skipped or cancelled !!");
                    PokktCustomNetworkVideoDelegate.onVideoSkipped(context, pokktConfig, network.getName());
                } else {
                    Log.d(TAG, "onAdColonyAdAttemptFinished ad is completely watched !!");
                    PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
                }
                Log.d(TAG, "onAdColonyAdAttemptFinished ad is closed now !!");
                PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
            } else if (ad.notShown()) {
                String reason = "Video Download Error";
                if (ad.noFill()) {
                    reason = "No Campaigns from AdColony";
                }

                Log.d(TAG, "Could not show ad from AdColony for reason: " + reason);
                v4vc_ad = null;
            }
        } catch (Exception e) {
            Log.e(TAG, "Ad Attempt failed");
            v4vc_ad = null;
        }
    }

    @Override
    // Ad Started Callback - called only when an ad successfully starts playing
    public void onAdColonyAdStarted(AdColonyAd ad) {
        Log.d(TAG, "onAdColonyAdStarted called!!");

        PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
    }

    @Override
    // Reward Callback
    public void onAdColonyV4VCReward(AdColonyV4VCReward adColonyV4VCReward) {
        Log.d(TAG, "onAdColonyV4VCReward called!!");
        try {
            if (adColonyV4VCReward.success()) {
                Log.d(TAG, "onAdColonyV4VCReward is success !!");
                Log.d(TAG, "onAdColonyV4VCReward video gratified is gettig called !!");
                PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, adColonyV4VCReward.amount(), network.getName());
            }
        } catch (Exception e) {
            Log.e(TAG, "Gratify failed");
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
        Log.d(TAG, "activity created callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Created AdColony");
                AdColony.addAdAvailabilityListener(this);
                AdColony.addV4VCListener(this);
            }
        } catch (Exception e) {
            Log.e(TAG, "Create failed");
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {
        Log.d(TAG, "activity started callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivityResumed(Activity activity) {
        Log.d(TAG, "activity resume callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Resumed AdColony");
                AdColony.resume(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, "Resume failed");
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        Log.d(TAG, "activity pause callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Paused AdColony");
                AdColony.pause();
            }
        } catch (Exception e) {
            Log.e(TAG, "Pause failed");
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
        Log.d(TAG, "activity stopped callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
        Log.d(TAG, "activity save instance callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        Log.d(TAG, "activity destroyed callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Destroyed AdColony");
                AdColony.cancelVideo();
                AdColony.removeAdAvailabilityListener(this);
                AdColony.removeV4VCListener(this);
            }
        } catch (Exception e) {
            Log.e(TAG, "Cleanup failed");
        }
    }
}
