package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.PokktManager;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.applovin.adview.AppLovinIncentivizedInterstitial;
import com.applovin.sdk.AppLovinAd;
import com.applovin.sdk.AppLovinAdClickListener;
import com.applovin.sdk.AppLovinAdDisplayListener;
import com.applovin.sdk.AppLovinAdLoadListener;
import com.applovin.sdk.AppLovinAdRewardListener;
import com.applovin.sdk.AppLovinAdVideoPlaybackListener;
import com.applovin.sdk.AppLovinErrorCodes;
import com.applovin.sdk.AppLovinSdk;

import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

public class AppLovinNetwork implements AdNetwork, AppLovinAdRewardListener, AppLovinAdVideoPlaybackListener,
        AppLovinAdDisplayListener, AppLovinAdLoadListener, AppLovinAdClickListener, Application.ActivityLifecycleCallbacks {

    private static final String TAG = AppLovinNetwork.class.getName();
    private Context context = null;
    private Activity launcherActivity = null;
    private PokktConfig pokktConfig = null;
    private AppLovinIncentivizedInterstitial incentivizedInterstitial;
    private boolean isInitialized = false;
    private float amount;
    private Network network = null;

    private boolean isTimedOut = false;
    private boolean isAdAvailableSinceLastCall = false;
    private Timer timer;

    @Override
    public void initNetwork(final Context context, final PokktConfig pokktConfig, final Network network) throws Exception {
        Log.i(TAG, "AppLovin init network called!!");
        this.pokktConfig = pokktConfig;

        synchronized (AppLovinNetwork.class) {
            if (isInitialized) {
                Log.i(TAG, "AppLovin init network already initialized!!");
                return;
            }
            this.context = context;
            this.network = network;
            launcherActivity = ((Activity) context);
            launcherActivity.getApplication().registerActivityLifecycleCallbacks(this);
            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey("amount")) {
                amount = Float.parseFloat(network.getCustomData().get("amount"));
            } else {
                throw new Exception("AppLovin Init Configurations Error!");
            }
            AppLovinSdk.initializeSdk(context);
            Log.i(TAG, "AppLovin init network initialized !!");
            isInitialized = true;
        }
    }

    @Override
    public void cacheVideo(final Network network) {
        Log.i(TAG, "Cache video called!!");
        if (isAdAvailableSinceLastCall) {
            isAdAvailableSinceLastCall = false;
            Log.i(TAG, "Video is cached since last call !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, amount, "0");
            return;
        }
        // Preload call using a AppLovinAdLoadListener
        try {
            if (incentivizedInterstitial == null) {
                Log.i(TAG, "Creating incentivized interstitial !!");
                incentivizedInterstitial = AppLovinIncentivizedInterstitial.create(context);
            }
            incentivizedInterstitial.preload(this);

            //startTimer
            startTimer();
        } catch (Exception e) {
            if(timer != null) {
                timer.cancel();
            }
            isAdAvailableSinceLastCall = false;
            Log.e(TAG,e.getMessage(),e);
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Cache Video Failed", network);
        }
    }

    private void startTimer() {
        isTimedOut = false;
        long duration = PokktManager.getTimeOutDuration(context, network);
        timer = new Timer(true);
        TimerTask timertask = new TimerTask() {
            @Override
            public void run() {
                isAdAvailableSinceLastCall = false;
                isTimedOut = true;
                String reason = "Time Out In Fetching Video";
                Log.e(TAG, reason);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            }
        };
        timer.schedule(timertask, duration);
    }

    @Override
    public void playVideo() {
        Log.i(TAG, "AppLovin play video called!!");
        try {
            if (incentivizedInterstitial.isAdReadyToDisplay()) {
                incentivizedInterstitial.show(launcherActivity, this, this, this, this);
            } else {
                Log.i(TAG, "No rewarded ad is currently available!!");
                throw new Exception("No Ad Available");
            }
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void adClicked(AppLovinAd appLovinAd) {
        Log.i(TAG, "AppLovin adClicked!");
    }

    @Override
    public void adDisplayed(AppLovinAd appLovinAd) {
        Log.i(TAG, "AppLovin adDisplayed !!");
    }

    @Override
    public void adHidden(AppLovinAd appLovinAd) {
        Log.i(TAG, "AppLovin adHidden !!");
        PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
    }

    @Override
    public void adReceived(AppLovinAd appLovinAd) {
        Log.i(TAG, "AppLovin adReceived !!");
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, amount, "0");
            if (timer != null) {
                timer.cancel();
            }
        } else {
            isAdAvailableSinceLastCall = true;
        }
    }

    @Override
    public void failedToReceiveAd(int i) {
        Log.i(TAG, "AppLovin failedToReceiveAd !" + i);
        String reason;
        switch (i) {
            case AppLovinErrorCodes.NO_FILL:
                reason = "No Campaigns from Applovin.";
                break;
            case AppLovinErrorCodes.FETCH_AD_TIMEOUT:
                reason = "Network conditions prevented the SDK from receiving an ad.";
                break;
            case AppLovinErrorCodes.INCENTIVIZED_NO_AD_PRELOADED:
                reason = "Requested for a rewarded video before one was available.";
                break;
            case AppLovinErrorCodes.INCENTIVIZED_SERVER_TIMEOUT:
                reason = "Reward validation requested timed out.";
                break;
            case AppLovinErrorCodes.INCENTIVIZED_UNKNOWN_SERVER_ERROR:
                reason = "Unknown server-side error.";
                break;
            case AppLovinErrorCodes.INCENTIVIZED_USER_CLOSED_VIDEO:
                reason = "User exited out of the video.";
                //may be dont gratify code
                break;
            case AppLovinErrorCodes.UNABLE_TO_PRECACHE_IMAGE_RESOURCES:
            case AppLovinErrorCodes.UNABLE_TO_PRECACHE_RESOURCES:
            case AppLovinErrorCodes.UNABLE_TO_PRECACHE_VIDEO_RESOURCES:
                reason = "Attempt to cache a resource to the filesystem failed.";
                break;
            case AppLovinErrorCodes.UNABLE_TO_RENDER_AD:
                reason = "Failure to render an ad on screen.";
                break;
            default:
                reason = "Unspecified Error !";
                break;
        }
        Log.i(TAG, "Could not show ad from AppLovin for reason: " + reason);
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            if (timer != null) {
                timer.cancel();
            }
        }
        incentivizedInterstitial = null;
    }

    @Override
    public void userRewardVerified(AppLovinAd appLovinAd, Map map) {
        Log.i(TAG, "AppLovin userRewardVerified !!");
        try {
            if (map.containsKey("amount")) {
                amount = Float.parseFloat((String) map.get("amount"));
            }
        } catch (Exception e) {
            Log.e(TAG, "Failed to parse amount");
        }
        PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, amount, network.getName());
    }

    @Override
    public void userOverQuota(AppLovinAd appLovinAd, Map map) {
        Log.i(TAG, "AppLovin userOverQuota!!");
    }

    @Override
    public void userRewardRejected(AppLovinAd appLovinAd, Map map) {
        Log.i(TAG, "userRewardRejected!!");
    }

    @Override
    public void validationRequestFailed(AppLovinAd appLovinAd, int i) {
        Log.i(TAG, "validationRequestFailed!!");
    }

    @Override
    public void userDeclinedToViewAd(AppLovinAd appLovinAd) {
        Log.i(TAG, "userDeclinedToViewAd!!");
    }

    @Override
    public void videoPlaybackBegan(AppLovinAd appLovinAd) {
        Log.i(TAG, "AppLovin videoPlaybackBegan !");
        PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
    }

    @Override
    public void videoPlaybackEnded(AppLovinAd appLovinAd, double percentViewed, boolean fullyWatched) {
        Log.i(TAG, "AppLovin videoPlaybackEnded!! " + "PercentageViewed :" + percentViewed + " fullywatched: " + fullyWatched);
        if (!fullyWatched) {
            PokktCustomNetworkVideoDelegate.onVideoSkipped(context, pokktConfig, network.getName());
        } else {
            PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        Log.d(TAG, "activity create callback for activity " + activity + " and launcher was " + launcherActivity);
        try {
            if (activity.getClass().getName().equals(launcherActivity.getClass().getName())) {
                Log.d(TAG, "Created AppLovin");
                launcherActivity = activity;
            }
        } catch (Exception e) {
            Log.e(TAG, "Create failed");
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {

    }

    @Override
    public void onActivityResumed(Activity activity) {

    }

    @Override
    public void onActivityPaused(Activity activity) {

    }

    @Override
    public void onActivityStopped(Activity activity) {

    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {

    }
}