package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.PokktManager;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.app.pokktsdk.util.Logger;
import com.app.pokktsdk.util.PokktStorage;
import com.chartboost.sdk.CBLocation;
import com.chartboost.sdk.Chartboost;
import com.chartboost.sdk.ChartboostDelegate;
import com.chartboost.sdk.Libraries.CBLogging;
import com.chartboost.sdk.Model.CBError;

import java.util.Timer;
import java.util.TimerTask;

public class ChartboostNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private String activityClassName = null;
    private Context context = null;
    private PokktConfig pokktConfig = null;

    public static final String APP_ID_KEY = "appId";
    public static final String APP_SIGNATURE = "appSignature";

    private static final String TAG = "POKKT_CHARTBOOST";

    public Network network;

    private boolean isInitialized = false;

    private boolean isTimedOut = false;
    private boolean isAdAvailableSinceLastCall = false;
    private Timer timer;

    @Override
    public void initNetwork(Context context, PokktConfig pokktConfig, Network network) throws Exception {

        this.pokktConfig = pokktConfig;

        synchronized (ChartboostNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "ChartBoostNetwork init network already initialized!!");
                return;
            }
            this.context = context;
            this.activityClassName = ((Activity) context).getClass().getName();
            this.network = network;
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

            String appId = "";
            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey(APP_ID_KEY)) {
                appId = network.getCustomData().get(APP_ID_KEY);
            } else {
                throw new Exception("ChartBoostNetwork Init Configurations Error!");
            }

            String appSignature = "";
            if (network.getCustomData().containsKey(APP_SIGNATURE)) {
                appSignature = network.getCustomData().get(APP_SIGNATURE);
            } else {
                throw new Exception("ChartBoostNetwork Init Configurations Error!");
            }

            Chartboost.startWithAppId(((Activity) context), appId, appSignature);
            Chartboost.setLoggingLevel(CBLogging.Level.ALL);
            Chartboost.setDelegate(delegate);
            Chartboost.onCreate(((Activity) context));
            Chartboost.onStart(((Activity) context));

            isInitialized = true;
        }
    }

    @Override
    public void cacheVideo(final Network network) {
        Log.d(TAG, "Cache Video Called");

        if (isAdAvailableSinceLastCall) {
            isAdAvailableSinceLastCall = false;
            Log.i(TAG, "Video is cached since last call !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
            return;
        }

        try {
            Chartboost.cacheRewardedVideo(CBLocation.LOCATION_DEFAULT);
            //start timer
            startTimer();
        } catch (Exception e) {
            Log.e(TAG,e.getMessage(),e);
            isAdAvailableSinceLastCall = false;
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Cache Video Failed", network);
        }
    }

    private void startTimer() {
        isTimedOut = false;
        long duration = PokktManager.getTimeOutDuration(context, network);
        timer = new Timer(true);
        TimerTask timertask = new TimerTask() {
            @Override
            public void run() {
                isAdAvailableSinceLastCall = false;
                isTimedOut = true;
                String reason = "Time Out In Fetching Video";
                Log.e(TAG, reason);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            }
        };
        timer.schedule(timertask, duration);
    }

    @Override
    public void playVideo() {
        Log.i(TAG,"Play Video Called");
        try {
            String toastStr = "Loading Rewarded Interstitial";
            if (Chartboost.hasRewardedVideo(CBLocation.LOCATION_DEFAULT))
                toastStr = "Loading Rewarded Interstitial From Cache";
            Log.i(TAG, toastStr);
            Chartboost.showRewardedVideo(CBLocation.LOCATION_DEFAULT);
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed", e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    private ChartboostDelegate delegate = new ChartboostDelegate() {

        //Called after a rewarded video has been loaded from the Chartboost API servers and cached locally.
        @Override
        public void didCacheRewardedVideo(String location) {
            Log.i(TAG, String.format("DID CACHE REWARDED VIDEO: '%s'", (location != null ? location : "null")));
            if (!isTimedOut) {
                isAdAvailableSinceLastCall = false;
                PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
                if (timer != null) {
                    timer.cancel();
                }
            } else {
                isAdAvailableSinceLastCall = true;
            }
        }

        //Called after a rewarded video has attempted to load from the ChartBoost API servers but failed.
        @Override
        public void didFailToLoadRewardedVideo(String location, CBError.CBImpressionError error) {
            Log.i(TAG, String.format("DID FAIL TO LOAD REWARDED VIDEO: '%s', Error:  %s", (location != null ? location : "null"), error.name()));
            Toast.makeText(context, String.format("FAILED TO LOAD REWARDED VIDEO '%s' because %s", location, error.name()), Toast.LENGTH_SHORT).show();
            if (!isTimedOut) {
                isAdAvailableSinceLastCall = false;
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, error.name(), network);
                if (timer != null) {
                    timer.cancel();
                }
            }
        }

        //
        @Override
        public void didFailToLoadInPlay(String location, CBError.CBImpressionError error) {
            Log.i(TAG, String.format("DID FAIL TO LOAD REWARDED VIDEO: '%s', Error:  %s", (location != null ? location : "null"), error.name()));
            Toast.makeText(context, String.format("FAILED TO LOAD REWARDED VIDEO '%s' because %s", location, error.name()), Toast.LENGTH_SHORT).show();
            if (!isTimedOut) {
                isAdAvailableSinceLastCall = false;
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, error.name(), network);
                if (timer != null) {
                    timer.cancel();
                }
            }
        }

        //Called after a rewarded video has been closed.
        @Override
        public void didCloseRewardedVideo(String location) {
            Log.i(TAG, String.format("DID CLOSE REWARDED VIDEO '%s'", (location != null ? location : "null")));
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
        }

        //Called after a rewarded video has been viewed completely and user is eligible for reward.
        @Override
        public void didCompleteRewardedVideo(String location, int reward) {
            Log.i(TAG, String.format("DID COMPLETE REWARDED VIDEO '%s' FOR REWARD %d", (location != null ? location : "null"), reward));
            PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
            PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, reward, network.getName());
        }

        //Called after a rewarded video has been displayed on the screen.
        @Override
        public void didDisplayRewardedVideo(String location) {
            Log.i(TAG, String.format("DID DISPLAY REWARDED VIDEO '%s' FOR REWARD", location));
            PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
        }
    };

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        Log.i(TAG, "Created activity "+activity+" with launcher "+activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Created ChartBoost");
                Chartboost.onCreate(activity);
                Chartboost.setDelegate(delegate);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {
        Log.i(TAG, "Started activity "+activity+" with launcher "+activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Started ChartBoost");
                Chartboost.onStart(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityResumed(Activity activity) {
        Log.i(TAG, "Resumed activity "+activity+" with launcher "+activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Resumed ChartBoost");
                Chartboost.onResume(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        Log.i(TAG, "Paused activity "+activity+" with launcher "+activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Paused ChartBoost");
                Chartboost.onPause(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
        Log.i(TAG, "Stopped activity "+activity+" with launcher "+activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Stopped ChartBoost");
                Chartboost.onStop(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        Log.i(TAG, "Destroyed activity "+activity+" with launcher "+activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Destroyed ChartBoost");
                Chartboost.onDestroy(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    public static boolean handleChartBoostBack() {
        Log.i(TAG, "Handle back press");
        return PokktStorage.getCurrentNetwork() != null && "ChartBoost".equalsIgnoreCase(PokktStorage.getCurrentNetwork().getName()) && Chartboost.onBackPressed();
    }

}
