package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.PokktManager;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.fusepowered.FuseError;
import com.fusepowered.FuseSDK;
import com.fusepowered.FuseSDKListener;
import com.fusepowered.IAPOfferInfo;
import com.fusepowered.RewardedInfo;
import com.fusepowered.VGOfferInfo;
import com.fusepowered.util.Player;

import java.util.ArrayList;
import java.util.Date;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

public class FusePoweredNetwork implements AdNetwork, FuseSDKListener, Application.ActivityLifecycleCallbacks {

    private static final String TAG = FusePoweredNetwork.class.getName();
    private String activityClassName = null;
    private Context context = null;
    private PokktConfig pokktConfig = null;
    private Network network = null;
    private String appId;
    private String zoneId;

    private static final String DEFAULT_APP_ID = "237bc8e4-e6ba-4fba-9ca0-3a8e4f5cd8a3";
    private static final String DEFAULT_ZONE_ID = "63612954";

    public static final String APP_ID_KEY = "appId";
    public static final String ZONE_ID_KEY = "zoneId";
    private boolean isInitialized = false;

    private boolean isTimedOut = false;
    private boolean isAdAvailableSinceLastCall = false;
    private Timer timer;

    @Override
    public void initNetwork(final Context context, final PokktConfig pokktConfig, final Network network) throws Exception {
        Log.d(TAG, "FusePowered init network called!!");
        this.pokktConfig = pokktConfig;

        synchronized (FusePoweredNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "FusePowered init network already initialized!!");
                return;
            }
            this.context = context;
            this.activityClassName = ((Activity) context).getClass().getName();
            this.network = network;
            if (network != null && network.getCustomData() != null && extrasAreValid(network.getCustomData())) {
                ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);
                Log.d(TAG, "FusePowered init network initialized !!");
                appId = network.getCustomData().get(APP_ID_KEY);
                zoneId = network.getCustomData().get(ZONE_ID_KEY);
                FuseSDK.startSession(appId, ((Activity) context), this, null);
                isInitialized = true;
            } else {
                throw new Exception("FusePowered Init Configurations Error!");
            }
        }
    }

    private boolean extrasAreValid(Map<String, String> extras) {
        return extras.containsKey(APP_ID_KEY) && extras.containsKey(ZONE_ID_KEY);
    }

    @Override
    public void cacheVideo(final Network network) {
        Log.d(TAG, "FusePowered cache video called!!");

        if (isAdAvailableSinceLastCall) {
            isAdAvailableSinceLastCall = false;
            Log.i(TAG, "FusePowered video is cached since last call !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
            return;
        }

        try {
            //start timer
            startTimer();
            FuseSDK.preloadAdForZoneID(zoneId);
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(),e);
            if (timer != null) {
                timer.cancel();
            }
            isAdAvailableSinceLastCall = false;
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Cache Video Failed", network);
        }
        Log.d(TAG, "FusePowered cache video finished!!");
    }

    private void startTimer() {
        isTimedOut = false;
        long duration = PokktManager.getTimeOutDuration(context, network);
        timer = new Timer(true);
        TimerTask timertask = new TimerTask() {
            @Override
            public void run() {
                isAdAvailableSinceLastCall = false;
                isTimedOut = true;
                String reason = "Time Out In Fetching Video";
                Log.e(TAG, reason);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            }
        };
        timer.schedule(timertask, duration);
    }

    @Override
    public void playVideo() {
        Log.d(TAG, "FusePowered play video called!!");
        FuseSDK.showAdForZoneID(zoneId, null);
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
        Log.d(TAG, "activity created callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivityStarted(Activity activity) {
        Log.d(TAG, "activity started callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivityResumed(Activity activity) {
        Log.d(TAG, "activity resume callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Resumed  Fuse Session");
                FuseSDK.resumeSession(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, "Resume failed");
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        Log.d(TAG, "activity pause callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Paused Fuse Session");
                FuseSDK.pauseSession();
            }
        } catch (Exception e) {
            Log.e(TAG, "Pause failed");
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
        Log.d(TAG, "activity stopped callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
        Log.d(TAG, "activity save instance callback for activity " + activity + " and launcher was " + activityClassName);
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        Log.d(TAG, "activity destroyed callback for activity " + activity + " and launcher was " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Ended Fuse Session");
                FuseSDK.endSession();
            }
        } catch (Exception e) {
            Log.e(TAG, "Resume failed");
        }
    }

    @Override
    public void sessionStartReceived() {
        Log.i(TAG, "sessionStartReceived");
    }

    @Override
    public void sessionLoginError(FuseError fuseError) {
        Log.i(TAG, "sessionLoginError");
    }

    @Override
    public void gameConfigurationReceived() {
        Log.i(TAG, "gameConfigurationReceived");

    }

    @Override
    public void accountLoginComplete(int i, String s) {
        Log.i(TAG, "accountLoginComplete");
    }

    @Override
    public void accountLoginError(String s, FuseError fuseError) {
        Log.i(TAG, "accountLoginError");
    }

    @Override
    public void notificationAction(String s) {
        Log.i(TAG, "notificationAction");
    }

    @Override
    public void notificationWillClose() {
        Log.i(TAG, "notificationWillClose");
    }

    @Override
    public void friendAdded(String s, FuseError fuseError) {
        Log.i(TAG, "friendAdded");
    }

    @Override
    public void friendRemoved(String s, FuseError fuseError) {
        Log.i(TAG, "friendRemoved");
    }

    @Override
    public void friendAccepted(String s, FuseError fuseError) {
        Log.i(TAG, "friendAccepted");
    }

    @Override
    public void friendRejected(String s, FuseError fuseError) {
        Log.i(TAG, "friendRejected");
    }

    @Override
    public void friendsMigrated(String s, FuseError fuseError) {
        Log.i(TAG, "friendsMigrated");
    }

    @Override
    public void friendsListUpdated(ArrayList<Player> arrayList) {
        Log.i(TAG, "friendsListUpdated");
    }

    @Override
    public void friendsListError(FuseError fuseError) {
        Log.i(TAG, "friendsListError");
    }

    @Override
    public void purchaseVerification(int i, String s, String s1) {
        Log.i(TAG, "purchaseVerification");
    }

    @Override
    public void adAvailabilityResponse(boolean isAvailable, int errorcode) {
        Log.i(TAG, "adAvailabilityResponse " + isAvailable + " errorcode " + errorcode);
        if (!isTimedOut) {
            if (timer != null) {
                timer.cancel();
            }
            isAdAvailableSinceLastCall = false;
            if (isAvailable) {
                PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
            } else {
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Error code " + errorcode, network);
            }
        } else {
            if (isAvailable) {
                isAdAvailableSinceLastCall = true;
            }
        }
    }

    @Override
    public void adWillClose() {
        Log.i(TAG, "adWillClose");
        PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
    }

    @Override
    public void adFailedToDisplay() {
        Log.i(TAG, "adFailedToDisplay");
        PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "adFailedToDisplay", network);
    }

    @Override
    public void adDidShow(int i, int i1) {
        Log.i(TAG, "adDidShow");
        PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
    }

    @Override
    public void rewardedAdCompleteWithObject(RewardedInfo rewardedInfo) {
        Log.d(TAG, "FusePowered rewarded !!");
        PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, (float) rewardedInfo.rewardAmount, network.getName());
    }

    @Override
    public void IAPOfferAcceptedWithObject(IAPOfferInfo iapOfferInfo) {

    }

    @Override
    public void virtualGoodsOfferAcceptedWithObject(VGOfferInfo vgOfferInfo) {

    }

    @Override
    public void timeUpdated(Date date) {

    }

    @Override
    public void didRecieveGCMRegistrationToken(String s) {

    }
}
