package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.PokktManager;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.fyber.Fyber;
import com.fyber.ads.AdFormat;
import com.fyber.currency.VirtualCurrencyErrorResponse;
import com.fyber.currency.VirtualCurrencyResponse;
import com.fyber.requesters.RequestCallback;
import com.fyber.requesters.RequestError;
import com.fyber.requesters.RewardedVideoRequester;
import com.fyber.requesters.VirtualCurrencyCallback;
import com.fyber.requesters.VirtualCurrencyRequester;

import java.util.Timer;
import java.util.TimerTask;

public class FyberNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks, RequestCallback, VirtualCurrencyCallback {

    public static final int REWARDED_VIDEO_REQUEST_CODE = 8796;

    public static final String TAG = "POKKT_FYBER";

    private static final String APP_ID = "AppKey";
    private static final String SECURITY_TOKEN = "security_token";

    private Context context = null;
    private PokktConfig pokktConfig = null;
    private String appId = "";
    private String securityToken = "";
    private boolean isInitialized = false;
    private Network network;
    private Intent intent;

    private boolean isTimedOut = false;
    private boolean isAdAvailableSinceLastCall = false;
    private Timer timer;

    public Intent getIntent() {
        return intent;
    }

    public void setIntent(Intent intent) {
        this.intent = intent;
    }

    public PokktConfig getPokktConfig() {
        return pokktConfig;
    }

    public void setPokktConfig(PokktConfig pokktConfig) {
        this.pokktConfig = pokktConfig;
    }


    @Override
    public void initNetwork(Context context, PokktConfig pokktConfig, Network network) throws Exception {
        this.pokktConfig = pokktConfig;

        synchronized (FyberNetwork.class) {
            if (isInitialized) {
                Log.e(TAG, "Fyber Network init network already initialized!!");
                return;
            }
            this.context = context;
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey(APP_ID)) {
                appId = network.getCustomData().get(APP_ID);
            } else {
                throw new Exception("Fyber Network Init Configurations Error!");
            }

            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey(SECURITY_TOKEN)) {
                securityToken = network.getCustomData().get(SECURITY_TOKEN);
            } else {
                throw new Exception("Fyber Network Init Configurations Error!");
            }

            try {

                // ** FYBER SDK INITIALIZATION ** //

                //when you start Fyber SDK you get a Settings object that you can use to customise the SDK behaviour.
                //Have a look at the method 'customiseFyberSettings' to learn more about possible customisation.
                Fyber.Settings fyberSettings = Fyber
                        .with(appId, ((Activity) context))
                        .withSecurityToken(securityToken)
                                // by default Fyber SDK will start precaching. If you wish to only start precaching at a later time you
                                // can uncomment this line and use 'CacheManager' to start, pause or resume on demand.
                                //.withManualPrecaching()
                                //if you do not provide an user id Fyber SDK will generate one for you
                        .withUserId(pokktConfig.getThirdPartyUserId())
                        .start();
                //customiseFyberSettings(fyberSettings);//uncomment to customise Fyber SDK

            } catch (IllegalArgumentException e) {
                Log.e(TAG, e.getLocalizedMessage());
            }
        }

        isInitialized = true;
    }

    //User Settings to customise Fyber SDK behaviour
    private void customiseFyberSettings(Fyber.Settings fyberSettings) {
        fyberSettings.notifyUserOnReward(false)
                .notifyUserOnCompletion(true);
    }

    @Override
    public void cacheVideo(final Network network) throws Exception {
        Log.i(TAG, "Cache Video Called");
        this.network = network;

        if (isAdAvailableSinceLastCall) {
            isAdAvailableSinceLastCall = false;
            Log.i(TAG, "Video is cached since last call !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
            return;
        }

        try {
            performRequest();
            //start timer
            startTimer();
        } catch (Exception e) {
            if (timer != null) {
                timer.cancel();
            }
            isAdAvailableSinceLastCall = false;
            Log.e(TAG,e.getMessage(),e);
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Cache Video Failed", network);
        }
    }

    private void startTimer() {
        isTimedOut = false;
        long duration = PokktManager.getTimeOutDuration(context, network);
        timer = new Timer(true);
        TimerTask timertask = new TimerTask() {
            @Override
            public void run() {
                isAdAvailableSinceLastCall = false;
                isTimedOut = true;
                String reason = "Time Out In Fetching Video";
                Log.e(TAG, reason);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            }
        };
        timer.schedule(timertask, duration);
    }

    @Override
    public void playVideo() {
        Log.i(TAG,"Play Video Called");
        try {
            if (intent != null) {
                Intent fyberIntent = new Intent(context, FyberActivity.class);
                fyberIntent.putExtra("networkName", network.getName());
                context.startActivity(fyberIntent);
            }
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    private VirtualCurrencyRequester getVirtualCurrencyRequester() {
        return VirtualCurrencyRequester.create(this)
                // user will get a toast notification upon reward
                .notifyUserOnReward(false)
//				this is the currency id for RV ad format
//				 you can refer to this -- http://developer.fyber.com/content/android/basics/rewarding-the-user/vcs/
//				.forCurrencyId("gold")
                ;
    }

    protected void performRequest() {
        //Requesting a rewarded video ad
        RewardedVideoRequester
                .create(this)
                        // you can add a virtual Currency Requester by chaining this extra method
                .withVirtualCurrencyRequester(getVirtualCurrencyRequester())
                .request(context);
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
    }

    @Override
    public void onActivityStarted(Activity activity) {
    }

    @Override
    public void onActivityResumed(Activity activity) {
    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityStopped(Activity activity) {
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    @Override
    public void onAdAvailable(Intent intent) {
        this.intent = intent;
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
        } else {
            isAdAvailableSinceLastCall = true;
        }
    }

    @Override
    public void onAdNotAvailable(AdFormat adFormat) {
        Log.i(TAG, adFormat.name());
        isAdAvailableSinceLastCall = false;
        if (!isTimedOut) {
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "No ad available", network);
        }
    }

    @Override
    public void onRequestError(RequestError requestError) {
        Log.e(TAG, "Something went wrong with the request: " + requestError.getDescription());
        isAdAvailableSinceLastCall = false;
        if (!isTimedOut) {
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, requestError.getDescription(), network);
        }
    }

    @Override
    public void onError(VirtualCurrencyErrorResponse virtualCurrencyErrorResponse) {
        Log.e(TAG, virtualCurrencyErrorResponse.getErrorMessage());
        isAdAvailableSinceLastCall = false;
        if (!isTimedOut) {
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, virtualCurrencyErrorResponse.getErrorMessage(), network);
        }
    }

    @Override
    public void onSuccess(VirtualCurrencyResponse virtualCurrencyResponse) {
        Log.i(TAG, "onRewardedVideoCompleted. Rewarded amount: " + virtualCurrencyResponse.getDeltaOfCoins());
        PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, (float) virtualCurrencyResponse.getDeltaOfCoins(), network.getName());
    }
}
