package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.PokktManager;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.mopub.common.MoPub;
import com.mopub.common.MoPubReward;
import com.mopub.mobileads.MoPubErrorCode;
import com.mopub.mobileads.MoPubRewardedVideoListener;

import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

public class MoPubNetwork  implements AdNetwork, Application.ActivityLifecycleCallbacks, MoPubRewardedVideoListener {

    private String activityClassName = null;
    private Context context = null;
    private PokktConfig pokktConfig = null;

    private static final String TAG = "POKKT_MOPUB";
    private static final String AD_UNIT_IT = "AdUnitId";
    private String adUnitId = "";
    private boolean isInitialized = false;
    private Network network;

    private boolean isTimedOut = false;
    private boolean isAdAvailableSinceLastCall = false;
    private Timer timer;

    @Override
    public void initNetwork(Context context, PokktConfig pokktConfig, Network network) throws Exception {
        this.pokktConfig = pokktConfig;

        synchronized (MoPubNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "MoPub Network init network already initialized!!");
                return;
            }
            this.context = context;
            this.activityClassName = ((Activity) context).getClass().getName();
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey(AD_UNIT_IT)) {
                adUnitId = network.getCustomData().get(AD_UNIT_IT);
            } else {
                throw new Exception("MoPub Network Init Configurations Error!");
            }

            isInitialized = true;
        }
        MoPub.initializeRewardedVideo(((Activity) context));
        MoPub.onCreate(((Activity) context));
        MoPub.setRewardedVideoListener(this);
    }

    @Override
    public void cacheVideo(final Network network) throws Exception {
        this.network = network;
        if (isAdAvailableSinceLastCall) {
            isAdAvailableSinceLastCall = false;
            Log.i(TAG, "Video is cached since last call !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
            return;
        }
        try {
            MoPub.loadRewardedVideo(adUnitId);//7491e5b2ca6b443da38f240c33b06db7

            //startTimer
            startTimer();
        } catch(Exception e) {
            isAdAvailableSinceLastCall = false;
            Log.e(TAG,e.getMessage(),e);
            if(timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, e.getMessage(), network);
        }
    }

    private void startTimer() {
        isTimedOut = false;
        long duration = PokktManager.getTimeOutDuration(context, network);
        timer = new Timer(true);
        TimerTask timertask = new TimerTask() {
            @Override
            public void run() {
                isAdAvailableSinceLastCall = false;
                isTimedOut = true;
                String reason = "Time Out In Fetching Video";
                Log.e(TAG, reason);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            }
        };
        timer.schedule(timertask, duration);
    }

        @Override
    public void playVideo() {
        try {
            if (MoPub.hasRewardedVideo(adUnitId))
                MoPub.showRewardedVideo(adUnitId);
        } catch(Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityStarted(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Started MoPub");
                MoPub.onStart(activity);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onActivityResumed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Resumed MoPub");
                MoPub.onResume(activity);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Paused MoPub");
                MoPub.onPause(activity);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Stopped MoPub");
                MoPub.onStop(activity);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Destroyed MoPub");
                MoPub.onDestroy(activity);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onRewardedVideoLoadSuccess(@NonNull String adUnitId) {
        Log.d(TAG, "onRewardedVideoLoadSuccess");
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
            if (timer != null) {
                timer.cancel();
            }
        } else {
            isAdAvailableSinceLastCall = true;
        }
    }

    @Override
    public void onRewardedVideoLoadFailure(@NonNull String adUnitId, @NonNull MoPubErrorCode errorCode) {
        Log.d(TAG, "onRewardedVideoLoadFailure: " + errorCode.name());
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, errorCode.toString(), network);
            if (timer != null) {
                timer.cancel();
            }
        }else{
            isAdAvailableSinceLastCall = false;
        }
    }

    @Override
    public void onRewardedVideoStarted(@NonNull String adUnitId) {
        Log.d(TAG, "onRewardedVideoStarted");
        PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
    }

    @Override
    public void onRewardedVideoPlaybackError(@NonNull String adUnitId, @NonNull MoPubErrorCode errorCode) {
        Log.d(TAG, "onRewardedVideoPlaybackError");
    }

    @Override
    public void onRewardedVideoClosed(@NonNull String adUnitId) {
        Log.d(TAG, "onRewardedVideoClosed");
        PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
    }

    @Override
    public void onRewardedVideoCompleted(@NonNull Set<String> adUnitIds, @NonNull MoPubReward reward) {
        Log.d(TAG, "onRewardedVideoCompleted. Rewarded amount: " + reward.getAmount());
        PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
        PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, reward.getAmount(), network.getName());
    }
}
