package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.tapjoy.TJActionRequest;
import com.tapjoy.TJConnectListener;
import com.tapjoy.TJEarnedCurrencyListener;
import com.tapjoy.TJError;
import com.tapjoy.TJGetCurrencyBalanceListener;
import com.tapjoy.TJPlacement;
import com.tapjoy.TJPlacementListener;
import com.tapjoy.TJVideoListener;
import com.tapjoy.TJViewListener;
import com.tapjoy.Tapjoy;
import com.tapjoy.TapjoyConnectFlag;

import java.util.Hashtable;

public class TapJoyNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks, TJGetCurrencyBalanceListener, TJPlacementListener {

    private static final String TAG = "POKKT_TAPJOY";

    private String activityClassName = null;
    private Context context = null;
    private PokktConfig pokktConfig = null;

    private static final String SDK_KEY = "sdkKey";

    private static String placementName = "";
    private static final String PLACEMENT_NAME = "placement_name";

    private Network network = null;
    private boolean isInitialized = false;

    private TJPlacement directPlayPlacement;

    @Override
    public void initNetwork(Context context, PokktConfig pokktConfig, Network network) throws Exception{
        this.pokktConfig = pokktConfig;

        synchronized (TapJoyNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "TapJoyNetwork init network already initialized!!");
                return;
            }
            this.context = context;
            this.activityClassName = ((Activity) context).getClass().getName();
            this.network = network;
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

            String sdkKey = "";
            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey(SDK_KEY)) {
                sdkKey = network.getCustomData().get(SDK_KEY);
            } else {
                throw new Exception("TapJoyNetwork Init Configurations Error!");
            }

            if(network.getCustomData().containsKey(PLACEMENT_NAME)) {
                placementName = network.getCustomData().get(PLACEMENT_NAME);
            } else {
                throw new Exception("TapJoyNetwork Init Configurations Error!");
            }

            isInitialized = true;

            Hashtable<String, Object> connectFlags = new Hashtable<String, Object>();
            connectFlags.put(TapjoyConnectFlag.ENABLE_LOGGING, "true");

            connectFlags.put(TapjoyConnectFlag.USER_ID, pokktConfig.getThirdPartyUserId());

            String tapjoySDKKey = sdkKey;
            //Tapjoy.setDebugEnabled(true);
            //Tapjoy.setGcmSender("34027022155");

            Tapjoy.connect(context.getApplicationContext(), tapjoySDKKey, connectFlags, new TJConnectListener() {
                @Override
                public void onConnectSuccess() {
                    TapJoyNetwork.this.onConnectSuccess();
                }

                @Override
                public void onConnectFailure() {
                    TapJoyNetwork.this.onConnectFail();
                }
            });
        }
    }

    @Override
    public void cacheVideo(final Network network) {
        Log.i(TAG,"Cache Video Called");
        try {
            if (directPlayPlacement != null) {
                if (directPlayPlacement.isContentAvailable() && directPlayPlacement.isContentReady())
                    PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
                else {
                    Log.i(TAG,"Content Not Ready");
                    PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "TapJoy Download Failed", network);
                }
            } else {
                Log.i(TAG,"Placement is null");
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "TapJoy Download Failed", network);
            }
        } catch(Exception e) {
            Log.e(TAG,e.getMessage(),e);
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "TapJoy download Failed", network);
        }
    }

    @Override
    public void playVideo() {
        Log.i(TAG,"Play Video Called");
        boolean playable = false;
        try {
            if (directPlayPlacement != null && directPlayPlacement.isContentAvailable()) {
                if (directPlayPlacement.isContentReady()) {
                    playable = true;
                }
            }
            if (playable) {
                directPlayPlacement.showContent();
            } else {
                PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
            }
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityStarted(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Started tapJoy");
                Tapjoy.onActivityStart(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityResumed(Activity activity) {

    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityStopped(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Stopped TapJoy");
                Tapjoy.onActivityStop(activity);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    /**
     * Handles a successful connect to Tapjoy. Pre-loads direct play placement
     * and sets up Tapjoy listeners
     */
    public void onConnectSuccess() {
        directPlayPlacement = new TJPlacement(context, placementName, this);
        directPlayPlacement.requestContent();
        Tapjoy.setEarnedCurrencyListener(new TJEarnedCurrencyListener() {
            @Override
            public void onEarnedCurrency(String currencyName, int amount) {
                //Nothing to do here
                Log.i(TAG, "Tapjoy Earned "+ amount + " " +currencyName);
                PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, amount, network.getName());
            }
        });

        // Setup listener for Tapjoy view callbacks
        Tapjoy.setTapjoyViewListener(new TJViewListener() {
            @Override
            public void onViewWillOpen(int viewType) {
                //Nothing to do here
            }

            @Override
            public void onViewWillClose(int viewType) {
                //Nothing to do here
            }

            @Override
            public void onViewDidOpen(int viewType) {
                //Nothing to do here
            }

            @Override
            public void onViewDidClose(int viewType) {
                Tapjoy.getCurrencyBalance(TapJoyNetwork.this);
                PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
            }
        });

        // Setup listener for Tapjoy video callbacks
        Tapjoy.setVideoListener(new TJVideoListener() {
            @Override
            public void onVideoStart() {
                Log.i(TAG, "video has started");
                PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
            }

            @Override
            public void onVideoError(int statusCode) {
                //If in any case video doesn't play then trigger the download failed event.
                Log.i(TAG, "there was an error with the video: " + statusCode);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "TapJoy download failed", network);
            }

            @Override
            public void onVideoComplete() {
                Log.i(TAG, "video has completed");

                // Best Practice: We recommend calling getCurrencyBalance as often as possible so the user�s balance is always up-to-date.
                Tapjoy.getCurrencyBalance(TapJoyNetwork.this);
                PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
            }
        });
    }

    /**
     * Handles a failed connect to Tapjoy
     */
    public void onConnectFail() {
        Log.e(TAG, "Tapjoy connect call failed");
        PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "TapJoy connection failed", network);
    }

    @Override
    public void onGetCurrencyBalanceResponse(String s, int rewardAmount) {
        //do nothing here
        Log.i(TAG,s+" " + rewardAmount);
    }

    @Override
    public void onGetCurrencyBalanceResponseFailure(String error) {
        //Nothing to do here
    }

    @Override
    public void onRequestSuccess(TJPlacement placement) {
        // If content is not available you can note it here and act accordingly as best suited for your app
        directPlayPlacement = placement;
        Log.i(TAG, "Tapjoy on request success, contentAvailable: " + placement.isContentAvailable());
    }

    @Override
    public void onRequestFailure(TJPlacement tjPlacement, TJError error) {
        try {
            Log.e(TAG, error.toString());
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, error.toString(), network);
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onContentReady(TJPlacement placement) {
        Log.d(TAG, "onContentReady");
        directPlayPlacement = placement;
    }

    @Override
    public void onContentShow(TJPlacement placement) {
        Log.d(TAG, "onContentShow");
    }

    @Override
    public void onContentDismiss(TJPlacement placement) {
        // Begin preloading the next placement after the previous one is dismissed
        directPlayPlacement = new TJPlacement(context, placementName, this);
        directPlayPlacement.requestContent();
    }

    @Override
    public void onPurchaseRequest(TJPlacement placement, TJActionRequest tjActionRequest, String s) {
        Log.d(TAG, "onPurchaseRequest");
    }

    @Override
    public void onRewardRequest(TJPlacement placement, TJActionRequest request, String itemId, int quantity) {
        Log.d(TAG, "onRewardRequest");
    }
}
