package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;

import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.app.pokktsdk.util.Logger;
import com.unity3d.ads.android.IUnityAdsListener;
import com.unity3d.ads.android.UnityAds;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by pankajbarnwal on 16/10/15.
 */
public class Unity3DAdNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks, IUnityAdsListener {

    //private Context context = null;
    private Activity launcherActivity = null;
    private PokktConfig pokktConfig = null;

    private static final String TAG = Unity3DAdNetwork.class.getName();
    private boolean isInitialized = false;
    private Network network;

    private static final String GAME_ID = "gameId";
    private String gameId = "";

    private String gamersIdKey = "";

    private static final String REWARD_AMOUNT = "rewardAmount";
    private String rewardAmount = "";

    @Override
    public void initNetwork(Context context, PokktConfig pokktConfig, Network network) throws Exception {
        launcherActivity = (Activity)context;
        this.pokktConfig = pokktConfig;

        synchronized (Unity3DAdNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "Unity3DAdNetwork init network already initialized!!");
                return;
            }
            this.network = network;
            ((Activity)context).getApplication().registerActivityLifecycleCallbacks(this);

            if (network != null && network.getCustomData() != null && extrasAreValid(network.getCustomData())) {
                gameId = network.getCustomData().get(GAME_ID);
                rewardAmount = network.getCustomData().get(REWARD_AMOUNT);
                Logger.d("Received data from server are GameId: " + gameId + ", RewardAmount:" + rewardAmount);
            } else {
                throw new Exception("Unity3DAdNetwork Init Configurations Error!");
            }

            gamersIdKey = pokktConfig.getThirdPartyUserId();

            isInitialized = true;
        }

        UnityAds.setDebugMode(true);
        //UnityAds.setTestMode(true);

        UnityAds.init(((Activity)context), gameId, this);
        UnityAds.setListener(this);
    }

    private boolean extrasAreValid(Map<String, String> extras) {
        return extras.containsKey(GAME_ID) && extras.containsKey(REWARD_AMOUNT);
    }

    @Override
    public void cacheVideo(Network network) throws Exception {
        if (UnityAds.canShow()) {
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(launcherActivity, pokktConfig, network, Float.parseFloat(rewardAmount), "0");
        } else {
            PokktCustomNetworkVideoDelegate.onDownloadFailed(launcherActivity, pokktConfig, "Unity3DAd download failed", network);
        }
    }

    @Override
    public void playVideo() {
        try {
            if (UnityAds.canShow()) {
                Map<String, Object> optionsMap = new HashMap<>();
                optionsMap.put(UnityAds.UNITY_ADS_OPTION_NOOFFERSCREEN_KEY, true);
                optionsMap.put(UnityAds.UNITY_ADS_OPTION_OPENANIMATED_KEY, false);
                optionsMap.put(UnityAds.UNITY_ADS_OPTION_GAMERSID_KEY, gamersIdKey);
                optionsMap.put(UnityAds.UNITY_ADS_OPTION_MUTE_VIDEO_SOUNDS, false);
                optionsMap.put(UnityAds.UNITY_ADS_OPTION_VIDEO_USES_DEVICE_ORIENTATION, false);
                UnityAds.setZone("rewardedVideoZone");
                UnityAds.show(optionsMap);
            }
        }catch(Exception e){
            Log.e(TAG, e.getMessage(), e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(launcherActivity, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        Log.d(TAG, "activity create callback for activity " + activity + " and launcher was " + launcherActivity);
        try {
            if (activity.getClass().getName().equals(launcherActivity.getClass().getName())) {
                Log.d(TAG, "Created Unity3DAdNetwork");
                launcherActivity = activity;
            }
        } catch (Exception e) {
            Log.e(TAG, "Create failed");
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {
    }

    @Override
    public void onActivityResumed(Activity activity) {
        try {
            if (activity.equals(launcherActivity))
                UnityAds.changeActivity(launcherActivity);
                UnityAds.setListener(this);
        } catch(Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityStopped(Activity activity) {
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    @Override
    public void onHide() {
        PokktCustomNetworkVideoDelegate.onVideoClosed(launcherActivity, pokktConfig, false, network.getName());
    }

    @Override
    public void onShow() {
        //not used for us
    }

    @Override
    public void onVideoStarted() {
        PokktCustomNetworkVideoDelegate.onVideoDisplayed(launcherActivity, pokktConfig, network.getName());
    }

    @Override
    public void onVideoCompleted(String rewardItemKey, boolean skipped) {
        if(skipped) {
            PokktCustomNetworkVideoDelegate.onVideoSkipped(launcherActivity, pokktConfig, network.getName());
        }
        else {
            PokktCustomNetworkVideoDelegate.onVideoCompleted(launcherActivity, pokktConfig, network.getName());
            PokktCustomNetworkVideoDelegate.onVideoGratified(launcherActivity, pokktConfig, Float.parseFloat(rewardAmount), network.getName());
        }
    }

    @Override
    public void onFetchCompleted() {
        Log.d(TAG, "onFetchCompleted");
    }

    @Override
    public void onFetchFailed() {
        Log.d(TAG, "onFetchFailed");
    }
}
