package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.vungle.publisher.AdConfig;
import com.vungle.publisher.EventListener;
import com.vungle.publisher.VunglePub;

public class VungleNetwork implements AdNetwork, EventListener, Application.ActivityLifecycleCallbacks {

    private static final String TAG = VungleNetwork.class.getName();
    private Context context = null;
    private String targetClassname = null;
    private PokktConfig pokktConfig = null;
    private boolean isInitialized = false;
    private Network network = null;
    final VunglePub vunglePub = VunglePub.getInstance();

    private boolean isAdAvailable = false;

    @Override
    public void initNetwork(final Context context, final PokktConfig pokktConfig, final Network network) throws Exception {
        Log.i(TAG, "Vungle init network called!!");
        this.pokktConfig = pokktConfig;

        synchronized (VungleNetwork.class) {
            if (isInitialized) {
                Log.i(TAG, "Vungle init network already initialized!!");
                return;
            }
            this.context = context;
            this.network = network;
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);
            String appId;
            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey("appId")) {
                appId = network.getCustomData().get("appId");
            } else {
                throw new Exception("Vungle Init Configurations Error!");
            }
            // initialize the Publisher SDK
            vunglePub.init(context, appId);
            vunglePub.addEventListeners(this);
            Log.i(TAG, "Vungle init network initialized !!");
            isInitialized = true;
        }
    }

    @Override
    public void cacheVideo(final Network network) {
        Log.i(TAG, "Cache video called!!");
        if (isAdAvailable) {
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
        } else {
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Cache Video Failed", network);
        }
    }

    @Override
    public void playVideo() {
        Log.i(TAG, "Vungle play video called!!");
        try {
            // create a new AdConfig object
            final AdConfig overrideConfig = new AdConfig();
            // set incentivized option on
            overrideConfig.setIncentivized(true);
            overrideConfig.setIncentivizedUserId(pokktConfig.getThirdPartyUserId());
            overrideConfig.setIncentivizedCancelDialogTitle("Careful!");
            overrideConfig.setIncentivizedCancelDialogBodyText("If the video isn't completed you won't get your reward! Are you sure you want to close early?");
            overrideConfig.setIncentivizedCancelDialogCloseButtonText("Close");
            overrideConfig.setIncentivizedCancelDialogKeepWatchingButtonText("Keep Watching");
            // the overrideConfig object will only affect this ad play.
            vunglePub.playAd(overrideConfig);
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed", e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Log.d(TAG, "Created Vungle");
                vunglePub.addEventListeners(this);
            }
        } catch (Exception e) {
            Log.e(TAG, "Create failed");
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {

    }

    @Override
    public void onActivityResumed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Log.d(TAG, "Resumed Vungle");
                vunglePub.onResume();
            }
        } catch (Exception e) {
            Log.e(TAG, "Resume failed");
        }

    }

    @Override
    public void onActivityPaused(Activity activity) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Log.d(TAG, "Paused Vungle");
                vunglePub.onPause();
            }
        } catch (Exception e) {
            Log.e(TAG, "Pause failed");
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {

    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Log.d(TAG, "Destroyed Vungle");
                vunglePub.removeEventListeners(this);
            }
        } catch (Exception e) {
            Log.e(TAG, "Destroyed failed");
        }
    }

    @Override
    public void onAdEnd(boolean b) {
        Log.i(TAG, "Vungle adHidden !!");
        PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());

    }

    @Override
    public void onAdStart() {
        Log.i(TAG, "Vungle videoPlaybackBegan !");
        PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
    }

    @Override
    public void onAdUnavailable(String reason) {
        Log.i(TAG, "Failed to receive ad from Vungle for reason: " + reason);
        isAdAvailable = false;
        PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
    }

    @Override
    public void onAdPlayableChanged(boolean isAdPlayable) {
        Log.i(TAG, "Vungle Ad Available " + isAdPlayable);
        isAdAvailable = isAdPlayable;
    }

    @Override
    public void onVideoView(boolean isCompletedView, int watchedMillis, int videoDurationMillis) {
        Log.i(TAG, "Vungle videoPlaybackEnded!! " + "Duration Viewed :" + watchedMillis + " fullywatched: " + isCompletedView);
        if (isCompletedView) {
            PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
        } else {
            PokktCustomNetworkVideoDelegate.onVideoSkipped(context, pokktConfig, network.getName());
        }

    }
}